.ps 58,80
.lm 8.rm 72
.nhy.nnm
.hd mixed
.c ;The Decus C Language System
.c ;Second Decus Release
.s 2
.c ;By
.s 2
.c ;David G. Conroy
.c ;Martin Minow
.c ;Robert B. Denny
.c ;Charles Forsyth
.s 3
.c ;Submitted by:
.s
.c ;Martin Minow
.s
.c ;Digital Equipment Corporation
.c ;1945 Andover St.
.c ;Tewksbury MA, 01876, U.S.A.
.s 4
.f
This document describes the changes and additions to the Decus C system
since the first submission of Decus C in November 1980.  There are
a few places in which the current compiler and library are
incompatible with the previous release, which will require conversion
of user programs.
.hl 1 Distribution
The original distribution contained a very large number of -- mostly --
small files.  The new distribution uses a text archive program to combine
infrequently-used components into a smaller number of larger files.
.s
The format of the archive "container" file was carefully chosen to
be suitable for distribution:  there are no record counts or embedded
binary data.  While this makes the archiver unsuitable for more general
usage, it is possible for the user to unpack or edit the archive with
any text editor.  The archive container file format will be described
in a subsequent section.
.s
Command files for C programs are now generated by a program, BUILD,
that understands conventions for the various operating systems supported
by Decus C.
.hl 1 Compiler
There have been many changes and improvements to the compiler:
.list "o"
.le ;In-line EIS (multiply and divide) instructions are supported.
.le ;All floating-point operations are supported.  While the compiler
generates code for systems lacking floating-point (FPU) hardware,
the library routines are non-existant.
.le ;A psect directive is supported allowing the programmer (almost) full
control over program sections that are generated.
.le ;The compiler syntax checking is more rigorous, requiring changes
to programs that were written using out of date C compilers.  For
example, previous versions of C allowed a pointer variable to point
to any structure member.  Now, the variable may only point to members
in the structure to which it points.  For example:
.nf
	struct A {
		int	a1, a2;
	};

	struct B {
		int	b1, b2;
	};

	main() {
		struct A *ap;

		ap->b1 ...		/* Error	*/
		((struct B *)ap)->b1	/* Works	*/
.f
.le ;A warning message is printed for unreferenced local variables.
These may be caused by typographical errors or by removing code that
uses local variables without removing the variable definitions.  The
"-w" option turns off this error message.
.le ;Previous versions of the compiler generated ".globl" definitions
for all "extern" variables.  This version generates .globl's only
if the variable was actually used.  This is needed if the programmer
_#include's a function definition header file and wishes to build
some routines into an overlay segment.
.els
.hl 1 Standard Library
The library has been extensively expanded, including a display terminal
handler package modelled closely on routines in the Vms runtime library.
A few incompatible changes have been made which make Decus C more
compatible with the Unix and Vms libraries:
.list "o"
.le ;The "iovtoa" routine has been renamed "fgetname" and is compatible
with the routine of the same name in the Vax-11 C runtime library.
.le ;The rand routine is compatible with the Unix routine.  Irand
is compatible with the previous Decus C release:
.nf
	int irand(v)	returns 0 .. v-1
	long rand()	returns 0 .. (2<<32)-1
.f
.le ;The time routines have been redone.  time() now returns the
Unix time of day (seconds since Jan 1, 1970).  tod() is a new routine
that returns the time of day (since midnite today) that was returned
by previous versions of time().
.s
The ctime() routine is now compatible with Unix usage -- it takes a
time value as returned by time() rather than as returned by $$rtim().
.s
The rtime() routine is now called $$rtim.  The old ctime() routine
is now called $$ctim().  A new routine, $$utim() converts $$rtim
values to time() format.
.els
.hl 1 Other Libraries
There are several "native" libraries contributed by various people,
including a native RSTS/E i/o library and a library of general utility
functions.
.s
There is also an interface library to aid in running Decus C programs
under VMS.  It presupposes access to Vax-11C.
.hl 1 Usage
On RSTS, the MCR command is now used by the operating system.
Consequently, MCR has now been renamed CRUN.
.hl 1 Tools
Various new tools including a text archiver (AR), a command file
builder (BUILD), and a file typeout program (T).
.s
All tools compile correctly using Vax-11C.  Most work compatibly, while
some have added functionality.  A few are so system specific that,
while they might compile on Vax-11 C, they probably don't do anything
useful.
.hl 1 Other Stuff
Due to space considerations, the large cookie file (now grown to over
11,000 entries) is no longer distributed with Decus C.  Instead, there
is a text justifier (ROFF) based on the program described in Software Tools
and several other programs, such as a units conversion routine.
.s
Note that many programs that have not been used (by the developers)
have not been tested and probably won't work correctly.
.hl 1 Documentation
The getrno program has been enhanced to allow C program documentation
to be embedded in source code.  Unlike the format used by the run-time
library, it is possible for the programmer to include runoff commands
in the tool documentation.  The (general) format of tool source
is as follows:
.s.nf
	/*
	 * documentation starts here:
	 */
	_#ifdef	DOCUMENTATION
	title	tool__name	one__line__description
	index	tool__name	one__line__keyword__description
	subhead
		Justified text for this sub-head...
		.x -- runoff commands ...
	another_subhead
		...
	_#endif
	/*
	 * Documentation ends
	 */
.s.f
Note that this means that it is unwise to
.s
	_#define DOCUMENTATION
.s
.hl 1 Archive Tape Format
Files are combined into a "container" file using the following format:
.s.nf
	-h-	file__name	date__entered
	Text of the file
	-h-	next__file	date__entered
	...
.s.f
Within a file, a line beginning with '-' will be quoted by doubling the
'-':
.s.nf
	-h-	file__name	date__entered
	Text...
	--More text
	...
.s.f
This allows embedded archives to be supported.  To extract files from
an archive without access to AR,
use your favorite editor to search for the '-h-' header
and for the '-h-' of the next file (or the end of all files).
Then convert all instances of "--" at the beginning of a line
to "-".
